function c=densitycl(densities,K,varargin)
% DENSITYCL Classifier based on the class-conditional distribution
%
% This is the class densitycl constructor
%
% c=densitycl(densities,lambda,...)
% densitycL is a classifier object that represents the types of classifiers that try to
% approximate the density distribution of the components and class them according 
% to the most likely components.
% additional field 'p' is given (proportions of the components)
% and 'densities' : a cell array containing the K densities of the components.
% 
% densities can be only one Density object (instead of a cell array of Density objects), 
% so that each class is modelized with the same density.
% it can also be a string that define the intanciation of the Density
% objects
% 
% lambda parametrize the objective function beetween generative and
% discriminative classifier
%
% densitycl(...,'knownp',[0 or 1],...)
%       with knownp = 1, optimize the proportion parameters during learning,
%       with knownp = 0, do not optimize the proportion parameters.
% densitycl(...,'prop',p,...)
% give the vector of proportions if not optimized
% densitycl(...,'commonparams',str,...)
% defines the names of the parameters that are common for the densities.
% for example str='var' force the covariance matrices to be equals in
% gaussian densities, leading to Linear Discriminant Analysis (see LDA).
% Each Density object of the densitycl object must admit the parameter str
% as second parameter of the getparams method.
%
% See also DENSITY, MIXTDENS/SET, MIXTDENS/LEARN, DENSITYCL, 
%   CLUSTERING, UNSUPERVISED, STATLEARN

if nargin<1 %no argument
    densities = {gaussdens};
    K=1;
end

if nargin<2 %K is not specified
    if isnumeric(densities)
        K = densities;        
        densities = {gaussdens};
        densities = densities(ones(1,K));
    elseif iscell(densities)
        K = length(densities);
    else
        K=1;
    end
end

if nargin>=2 & isnumeric(K) & iscell(densities)
    num = length(densities);
    c=cell(1,num);
    for k=1:num
        c{k}=densitycl(densities{k},K,varargin);
    end
    return 
end


if ~isnumeric(K) % no K specified
    varargin={K,varargin{:}};
    K = length(densities);
end

%now the variables densities, K and varagin exist

%case of density not given in a cell array
if ~iscell(densities)
    densities = {densities};
    densities = densities(ones(1,max(K,1)));
end

%for parameters in a cell array
if length(varargin)>0
    if iscell(varargin{1})
        varargin = varargin{1};
    end
end

%check if multiple models are required

if iscell(densities{1});
    if length(densities)==1
        densities = densities(ones(1,max(K,1)));
    else
        K=length(densities);
    end
    num=zeros(1,K);
    for k=1:K
        num(k)=length(densities{k});
    end
    allcomb = allcombi(num);
    nmod = size(allcomb,1);
    c = cell(1,nmod);
    for i=1:nmod
        curdens = cell(1,K);
        for k=1:K
            curdens{k} = densities{k}{allcomb(i,k)};
        end
        c{i}=densitycl(curdens,varargin);
    end    
    return
end

if K~=0 & length(densities)~=K
    error('The number of distributions not not match the number of classes');
end

s = struct('p',ones(1,K)/K,'densities',[],'knownp',0,'commonparams',[]);
c = class(s,'densitycl',classifier);
c=setnbcl(c,K);
c.densities = densities;

if iscell(densities)
    c = setdim(c,getdim(densities{1}));
end

if length(varargin)>0
    c = set(c,varargin);
end
   
