function [lval,grad] = logvalue(c,x,cl)
% LOGVALUE - Log-density and parameters gradient of samples
%
% lval = LOGVALUE(c,x,cl) 
%   returns the log value of the density at points x for the correct group
%   specified in cl. If cl is absent or empty, then returns the logarithm 
%   of the sum of the densities weighted by the priors
%   lval is a column vector
% 
% [lval,grad] = LOGVALUE(c,x,cl) also returns the gradient according to
% parameters in a nxnp matrix (np in the number of parameters)
%
% [lval,grad] = LOGVALUE(c,x,-1) Split the computation into the different classes.
% It returns a nxK matrix for lval and a nxpxK matrix for grad 
% where n is the number of data points,  p the number of parameters and K the 
% number of classes. (this option is used by GDTFUNCTION)
%
% See also DENSITYCL/DERLOGLIKNUMERIC, DENSITYCL, DENSITY/LOGVALUE, 
%   DENSITY, DENSITY/DERLOGLIK, GAUSSDENS/DERLOGLIK
%   UNSUPERVISED, STATLEARN

splitclasses=0;

if nargin<3
    cl=[];
elseif cl==-1
    splitclasses=1;    
    cl=[];
end


K = getnbcl(c);
[n,d] = size(x);
if nargout<=1
    if isempty(cl)
        lval=zeros(n,K);
        for k=1:K
            lval(:,k)=c.p(k)*logvalue(c.densities{k},x);
        end
        if ~splitclasses
            lval = log(sum(exp(lval),2)+1e-320);
        end
    else
        lval=zeros(n,1);
        for k=1:K
            i = find(cl==k);
            lval(i)=logvalue(c.densities{k},x(i,:));
        end
    end
    
else% nargout>1
    if ~isempty(c.commonparams)
        error('gradient with common parameters not yet implemented');
    end
    if ~c.knownp
        error('The proportions must be known to compute the gradient');
    end
    np = zeros(1,K);
    for k=1:K
        np(k) = n_params(c.densities{k});
    end
    npcs = [0 cumsum(np)];
    nptot = npcs(end);
    if isempty(cl)
        lval=zeros(n,K);
        grad=zeros(n,nptot,K);
        for k=1:K
            [lval(:,k),grad(:,npcs(k)+1:npcs(k+1),k)]=logvalue(c.densities{k},x);
        end
        lval = lval + (ones(n,1)*log(c.p+1e-320));
        if ~splitclasses
            lval = log(sum(exp(lval),2)+1e-320);
            grad = sum(grad,3);
        end
    else
        error('not implemented yet');
        lval=zeros(n,1);
        for k=1:K
            i = find(cl==k);
            lval(i)=logvalue(c.densities{k},x(i,:));
        end
    end
end