function [v,grad]=gdtfunction(theta,model,X,y,lambda)
% GDTFUNCTION - Generative-Discriminative Tradeoff objective function
% v = GDTFUNCTION(params,densitycl,X,y,lambda)
% Returns the negative log-likelihood L_gen of the generative model if lambda=1
% or the negative conditional log-likelihood L_disc of the discriminative model 
% if lambda=0 (default). 
% If 0<lambda<1 then returns a weighted sum of the two:
%   v = lambda*L_gen + (1-lambda)/L_disc
% - params are the parameters that are set to model by the function
%   model = setparams(params) 
% - model is a classifier object having the method setparam defined, for example an 
% densitycl object.
% - X and y are the data and their class labels
% - lambda is the GDT parameter between 0 and 1
%
% [v,grad] = GDTFUNCTION(params,densitycl,X,y,lambda) also return the 
% gradient of the function according to the parameters.
% 
% Example:
% --------
% OPTIMISATION OF THE GDTFUNCTION
% [x,y]=rand(densitycl_example,300);
% cl0 = learn(densitycl({gaussdens,gaussdens}),x,y);
% cl0=set(cl0,'knownp',1);
% clf;subplot(211);show(cl0,x,y);
% 
% opts = optimset('GradObj','on','Display','iter','MaxIter',50);
% sol = fminunc('gdtfunction',getparams(cl0),opts,cl0,x,y,0.1);
% cl1= setparams(cl0,sol)
% subplot(212);show(setparams(cl1,sol),x,y);
% 
%
% See also GDTCL, DENSITYCL, FMINUNC, STATLEARN


model = setparams(model,real(theta));

if nargout<=1
    v = - loglik(model,X,y) + (1-lambda)*loglik(model,X);
else
    K=getnbcl(model);
    n=size(X,1);
    [lval,g] = logvalue(model,X,-1);
    %probas of classification
    pc = exp(lval);
    pc = pc./(sum(pc,2)*ones(1,K));
    ind = sub2ind([n,K],(1:n)',y);
    
    %value
    v = -lambda*(sum(lval(ind)))-(1-lambda)*sum(log(pc(ind)));
    
    %gradient
    om = -(1-lambda)*pc;    
    om(ind)=om(ind)+1;
    grad = -sum(sum(repmat(reshape(om,[n,1,K]),[1,size(g,2),1]).*g,3),1)';
    grad = real(grad);
end
v=real(v);

