function dist = distance(c1,c2,type,n)
% DISTANCE approximate the distance between 2 densities
% 
% dist = distance(P,Q)
% P and Q are densities (density objects)
% dist is a positive value equal to 0 if and only if the density are
% equal.
%
% dist = distance(P,Q,type) where type is a string specify the type of
% distance to use.
%
% dist = distance(P,carray) computes the distances between P and each
% density in the cell array carray. Returns a row vector.
% 
% types of distances:
% - 'KL' : Kullback-Leibler divergence (default). This is not truly a distance (not
% symetric) but this quantity is often used to mesure the adequation
% between 2 densities. It is equal to E[log(P(X)/Q(X))] when X follow 
% the P distribution. A monte-Carlo estimation is performed. (it implies
% that the computed quantity is random). The numbers of points simulated 
% can be specify in the 4th parameter.
% - symKL: symetric Kullback-Leibler distance. This is the symetric version of the
% previous quantity : .5*(E_P[log(P(X)/Q(X))]+E_Q[log(Q(X)/P(X))]).
%
% dist = distance(c1,c2,type,n)
% n is the number of simulations to perform to compute the mean.

if nargin<3
    type='KL';
end

if nargin<4
    n=10000;
end

if strcmp(class(c1),'density')
    nc=length(c2);
    for i=1:nc
        
    end
end

if ~iscell(c2)
    c2={c2};
end
nc=length(c2);

switch type
    case 'KL',
        x = rand(c1,n);
        dist = mean(log(value(c1,x)+eps)*ones(1,nc)-log(value(density,c2,x)+eps));
    case 'symKL',
        dist=zeros(1,nc);
        x1 = rand(c1,n);
        L1 = mean(log(value(c1,x1)+eps));
        for i=1:nc            
            x2 = rand(c2{i},n);
            dist(i) = .5*(L1 - mean(log(value(c2{i},x1)+eps))+...
                mean(log(value(c2{i},x2)+eps))-mean(log(value(c1,x2)+eps)));
        end
    otherwise        
        error('invalid distance name');
end
