function c= ML_learn(c,data,varargin)
%ML_LEARN Lean the Maximum Likelihood parameters by gradient ascent
%
% c = ML_LEARN(c,data) uses the FMINUNC function of the optimization toolbox
% to learn the parameters of the density.
% This function is like learn, but uses Quasi-Newtow method to find 
% optimal ML parameters.
%
% Initialization: 
% - if the dimension of the density (see DENSITY/GETDIM)
% is empty or different from the data dimension, the INIT method of the 
% density object is used to initialize the parameters
% of the object. If the initialization is exact (i.e. returns the true density,
% then the function returns the parameters without launching the Quasi-Newtown method.
% - if the dimension of the density matches the data, then the parameters of the 
% density are used for the initialization.
%
% Options:
%
% c= ML_LEARN(c,data,'display','iter') shows the iteration of the optimization procedure
% Other values set the level of display (See also OPTIMSET): off, iter, notify, final
% c= ML_LEARN(c,data,'gradient','off') does not use the gradient in 
% the optimization procedure, 
%
% For some densities, the gradient is too difficult to compute,
%
% c= ML_LEARN(c,data,'niter',Nmax) gives the maximal number of iterations,
%
% c= ML_LEARN(c,data,'epsi',e) gives in e the termination tolerance on the loglikelihood
%
% See also OPTIMSET, FMINUNC, DENSITY/LEARN
%

display = 'off';
gradient = 'on';
niter=100;
epsi = 1e-7;

%%% PARAMETERS MANAGEMENT
for i=1:2:length(varargin)
    if ~isstr(varargin{i}) | ~exist(varargin{i},'var')
        error(['Invalid parameter ' varargin{i}]);
    end
    eval([varargin{i} '= varargin{i+1};']);
end
%%%


options = optimset('GradObj',gradient,'Display',display,'MaxIter',niter,'TolFun',epsi);

d = getdim(c);
if isempty(d) | d~=size(data,2);
    c=init(c,data);
end
c = setdim(c,size(data,2));

%Optimization procedure
theta=fminunc('neglldensity',getparams(c),options,c,data);

c=setparams(c,theta);




