function h=show2(c,x,varargin)
%SHOW Plot the density contours on a 2D graph
%
% h=show2(c) displays the standard representation of the density:
% - the contour line containing 80% of the density
% - the special points specific to the density (for example, the clusters
% in a mixture distribution)
%
% h=show2(c,x) draws also the points defined in the matrix x
%
% h=show2(c,x,'paramsname',paramsval,...) defines the optional parameters that can be passed 
% as arguments.
%
% Available options [default value]:
% color [[.5 .5 .5]] %set the color of the density
% mode ['contour'] 
% lims [[]]
% factor [1] 
% proj_matrix [[]]
% range [[]]
% linewidth [1] 
% x[[]] data matrix to plot
% show_points[1 if x is specified] 
%    0 : do not draw the points in x matrix (but they are used to set the axis limits
%    1 : draw random points
% npoints [100] number of points to generate if x is not specified and show_points=1
% specialpoints [1] Draw the Special Parts (for examples clusters in mixtures)
% quantiles [[.8]] Set the levels of the isocontours
% SPLineWidth [1] Set le Linewidth of Special Parts (for examples clusters in mixtures)
% SPdraw_means [1] Draws the means of Special Parts
% axislabel[1] draw/remove the axis labels
%
% show(density,{c1,c2,...},x)
% multiple densities view in subplots
%
%See also SHOW DENSITY/SHOW DENSITY/SHOW3 DENSITY/SHOW1


color = [.5 .5 .5];
mode = 'contour'; 
lims = [];
factor = 1; 
proj_matrix = [];
range = [];
linewidth =1;
show_points=1;    
quantiles = [.8 ];
npoints =  20+ceil(10/(1.001-max(quantiles)));
specialpoints = 1;
SPLineWidth = 1;
SPdraw_means = 1;
margin=.2;
axeslabels = 1;

if isstr(x)
    x=[];
    varargin={x,varargin{:}};
elseif nargin>2 & iscell(varargin{1})
    varargin=varargin{1};
end

if length(varargin)>=1 & iscell(varargin{1})
    varargin = varargin{1};
end

%%% PARAMETERS MANAGEMENT
lineargs={};
for i=1:2:length(varargin)
    if ~isstr(varargin{i}) | ~exist(varargin{i},'var')
        lineargs = varargin(i:end);
        break
    end
    eval([varargin{i} '= varargin{i+1};']);
end
%%%

h=[];

if isempty(lims)
    if ~isempty(x)
        lims = [min(x);max(x)];
        lims = lims+margin*[-1;1]*diff(lims);
    else
        [m,v] = stats(c);
        lims = [1;1]*m' + 4*([-1;1]*sqrt(diag(v)'));
    end
else
    lims = reshape(lims,[2,2]);
end
 hold on
 if ~isempty(x)
    h = [h;line(x(:,1),x(:,2),'MarkerFaceColor',color,'Marker','s','MarkerSize',ceil(exp(1-size(x,1)/1000)),'LineStyle','none')];
end
    
nc = 50;
xp = (0:nc)/nc*(lims(2,1)-lims(1,1))+lims(1,1);
yp = (0:nc)/nc*(lims(2,2)-lims(1,2))+lims(1,2);
[mx,my] = meshgrid(xp,yp);
mz = reshape(value(c,[mx(:),my(:)]),size(mx));

switch(mode)
    case 'contour'
        if isempty(quantiles) | any(abs(.5-quantiles)>.5) 
            error('invalid quantiles (the values must be in [0,1])');
        end
        levels = value_at_quantile(c,quantiles,x); 
        if length(levels)==1
            levels=levels*[1,1];
        end
        [cont,hh]=contour(mx,my,mz,levels);
        h=[h;hh];
        grid off
        for i = 1:length(hh);
            hc = hh(i);
            fact = quantiles(min(i,length(quantiles)));
            set(hc,'EdgeColor',sqrt(color*fact));
            set(hc,'LineStyle','-','LineWidth',1);
            set(hc,lineargs(1:2:end),lineargs(2:2:end));
        end
        view(2);

    case 'surf'
        hh = surf(mx,my,mz);
        set(hh,'FaceColor',color);
        set(hh,'EdgeColor','none');
        set(hh,lineargs(1:2:end),lineargs(2:2:end));
        light('Position',[-10 0 0.01],'Color',[.4 .4 .4]);
        light('Position',[-10 -10 1],'Color',[.3 .3 .3]);
        lighting gouraud 
        %            shading('interp');        
        view(3);


    case 'image'
        ncmap=256;
        hh=imagesc(xp,yp,mz);
        set(hh,lineargs(1:2:end),lineargs(2:2:end));
        h=[h;hh];
        colormap(1-gray(ncmap).*(ones(ncmap,1)*(1-color)));
        
    case 'nocontour'
        view(2);
        hh = line(0,0,'Visible','off');
        set(hh,lineargs(1:2:end),lineargs(2:2:end));
        
    otherwise
        error('Unknown drawing mode');
end

if specialpoints
    [points,C,sz] = specialpoints(c);
    if isempty(C)
        h=[h;line(points(:,1),points(:,2),'LineStyle','none','Marker','o','MarkerSize',12,'MarkerFaceColor',1-(1-color)*.8,'MarkerEdgeColor',[0 0 0])];   
    else
        for s=1:size(C,3)
            var = C(:,:,s);
            multnormplot(points(s,:),var,'prop',.8,'draw_mean',SPdraw_means,'LineWidth',SPLineWidth,'Color',color*.8)
        end
    end
end  % special points

if ~axeslabels
    set(gca,'XTickLabel',[],'YTickLabel',[],'ZTickLabel',[]);
end

if nargout<1
    clear h
end