function c = learn(c,x,w,varargin)
% LEARN - Maximum Likelihood estimation with the EM algorithm
%   
% c = learn(c,x,w)
%
%	Find the parameters of the mixture thanks to the EM algorithm
%   c : MixtDens object
%   x : data matrix. Each row is a sample. d columns for each dimension
%   w : weights of each data
%
% c = learn(c,x,w,'niter',niter,...)
%      number max of iterations.(50 by default)
% c = learn(c,x,w,'niter',epsi,...)
%      tolerance on the likelihood for the stop criterion
% c = learn(c,x,w,...,'randominit',[1 or 0],...)
%        random initialization on/off
% c = learn(c,x,w,'partialclust',cl,...)
%       cl is a nxK matrix of partial clustering with Nan in the position of
%       unknown groups. K is the number of clusters
% c = learn(c,x,w,'display',[1 or 0],...)
%       display on/off
%   Output
%   c : MixtDens object
%
% See also MIXTDENS, DENSITY/LEARN, UNSUPERVISED, CLUSTERING, KMEANSLIGHT
%   STATLEARN

[n,d] = size(x);
c.density = setdim(c.density,d);

%weights
if nargin<3 | isempty(w)
   w = ones(n,1)/n; %equal weights
end

if ~isnumeric(w)
    varargin={w,varargin{:}};
    w = ones(n,1)/n; %equal weights    
end
    
%parameters
cl = [];
niter = 50;
K = getnbcl(c);
display = 2*(n*d>50000);
randominit = 1;
epsi = 1e-10;

%%% PARAMETERS MANAGEMENT
for i=1:2:length(varargin)
    if ~isstr(varargin{i}) | ~exist(varargin{i},'var')
        error(['Invalid parameter ' varargin{i}]);
    end
    eval([varargin{i} '= varargin{i+1};']);
end
%%%


%no cluster size specified
if K==0
    K = size(cl,2);
end

if length(c.densities) ~= K;
       error('incorrect number of clusters');
end

if K==1; %only 1 cluster
    c.p = 1;
    c.densities = c.densities(1);
    c.densities{1} = learn(c.densities{1},x,w);
    return
end

if ~isempty(cl); %
    error('partial clustering not implemented');   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZATION OF THE DENSITIES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d0=getdim(c.densities{1});
initparams = length(c.densities) ~= K | isempty(d0) | d0==0;    

if initparams %if the parameters need to be initialized
    [m,index] = kmeanslight(x,K);
	if ~c.knownp | isempty(c.p)
	    c.p = ones(1,K)/K;
	end
    for k=1:K
        c.densities{k} = learn(c.densities{k},x(find(index==k),:));
    end
else
    %nothing because the distribution is initialized
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


maxrestart = 5;
mincounts = 1+n_params(c.densities{1});
if mincounts*K>n & niter>1
    warning(['' num2str(n) ' data are not enough to learn a mixture of ' num2str(K) ' densities with ' num2str(n_params(c.densities{1})) ' parameters']);
    niter = min(5,niter);
end
minprop=mincounts/n;
%---------------------------------------------------------------------------------------------------------------------------------------------

it=1;
restart = 0;
conv=0;

%%%%%%%%%%%%%%%%%%
% MAIN LOOP OF EM
%%%%%%%%%%%%%%%%%%

while(it<niter) %beginning of the loop    
    %%%%%%%%%%%%%%%%%
    % M STEP
    %%%%%%%%%%%%%%%%%
    
    for k=1:K
        logom0(:,k) = log(c.p(k)) + logvalue(c.densities{k},x);
    end

    %log-likelihood  
    ll(it) = sum(log(sum(exp(logom0')+1e-320)));
    if display==2
        disp([num2str(it) ' ll = ' num2str(ll(it))]); %display the iterations of the EM algorithm
    end
    if (it>1 & abs(ll(it)-ll(it-1))<epsi*n) | restart>maxrestart;
        conv=1;
        break      
    end
    
    %normalization of the rows to obtain probabilities of belonging to each class for every data
    logom = logom0-log(repmat(sum(exp(logom0),2)+1e-320,1,K)); %normalize the lines

    %%%%%%%%%%%%%%%%%
    % M STEP
    %%%%%%%%%%%%%%%%%

    counts = sum(exp(logom));
	if ~c.knownp
	    c.p = max(minprop,counts/2/n);
	    c.p=c.p/sum(c.p);
	end
    
    for k=1:K
            if counts(k)<mincounts & restart <5 %if few data in a cluster                
                c.densities{k} = learn(c.densities{k},x,max(eps,-min(logom0,[],2)));
                restart = restart+1;
                it=1;
                %counts(k+1:end)=counts(k+1:end)+mincounts;
                break
            else
                c.densities{k} = learn(c.densities{k},x,exp(logom(:,k)));
            end                
    end
    
    if ~isempty(c.commonparams)  %dependents parameters       
        c.densities = mixcommonparams(c.densities{1},c.densities(2:end),c.commonparams,c.p);
    end

    it=it+1;
end 


if display
    %fprintf('d=%d,n=%d,K=%d==>np=%d\n',d,n,K,n_params(c));
    if restart>maxrestart
        disp(['EM cannot convergence due to infinite maximum likelihood (' num2str(K) ' groups)']);        
    elseif conv 
        disp(['EM convergence in ' num2str(it-1) ' iterations (' num2str(K) ' groups)']);
    else
        disp(['EM stopped before convergence in ' num2str(niter) ' iterations (' num2str(K) ' groups)']);
    end
end
