function c = learn(c,x,L,varargin)
% LEARN - Cluster Repartition Constraint on the EM algorithm
%   
% c = learn(c,x,w)
%
%	Find the parameters of the mixture thanks to the EM algorithm
%   c : MixtDens object
%   x : data matrix. Each row is a sample. d columns for each dimension
%   L : cluster repartition constraint. Vector of labels representing groups of data:
%       For a given group, the number of elements in each cluster cannot be greater than 1.
%       This constraint forces the clusters to cover as many groups as possible
%
% c = learn(c,x,L,'nbmax',a,...) set the maximal number of elements of one group in each
%   clusters to the scalar a instead of 1
%
% c = learn(c,x,L,'niter',niter,...)
%      number max of iterations.(50 by default)
% c = learn(c,x,L,'niter',epsi,...)
%      tolerance on the likelihood for the stop criterion
% c = learn(c,x,L,...,'randominit',[1 or 0],...)
%        random initialization on/off
% c = learn(c,x,L,'display',[1 or 0],...)
%       display on/off
%   Output
%   c : MixtDens object
%
% See also MIXTDENS, DENSITY/LEARN, UNSUPERVISED, CLUSTERING, KMEANSLIGHT
%   STATLEARN

[n,d] = size(x);
c.density = setdim(c.density,d);

%weights
if nargin<3 | isempty(L)
   L = (1:n)';
end

   
%parameters
cl = [];
niter = 50;
K = getnbcl(c);
display = 2*(n*d>50000);
randominit = 1;
epsi = 1e-10;
nbmax = 1;

%%% PARAMETERS MANAGEMENT
for i=1:2:length(varargin)
    if ~isstr(varargin{i}) | ~exist(varargin{i},'var')
        error(['Invalid parameter ' varargin{i}]);
    end
    eval([varargin{i} '= varargin{i+1};']);
end
%%%


%no cluster size specified
if K==0
    K = size(cl,2);
end

if length(c.densities) ~= K;
       error('incorrect number of clusters');
end

if K==1; %only 1 cluster
    c.p = 1;
    c.densities = c.densities(1);
    c.densities{1} = learn(c.densities{1},x,w);
    return
end

if ~isempty(cl); %
    error('partial clustering not implemented');   
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIALIZATION OF THE DENSITIES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d0=getdim(c.densities{1});
initparams = length(c.densities) ~= K | isempty(d0) | d0==0;    

if initparams %if the parameters need to be initialized
    [m,index] = kmeanslight(x,K);
    c.p = ones(1,K)/K;
    for k=1:K
        c.densities{k} = learn(c.densities{k},x(find(index==k),:));        
    end
else
    %nothing because the distribution is initialized
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indsL = lbl2cell(L);
numL = cellfun('length',indsL);
NL = length(indsL);;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


maxrestart = 5;
mincounts = NL/10;
if mincounts*K>n
    warning(['' num2str(n) ' data are not enough to learn a mixture of ' num2str(K) ' densities with ' num2str(n_params(c.densities{1})) ' parameters']);
    niter = 2;
end
minprop=mincounts/n;
%---------------------------------------------------------------------------------------------------------------------------------------------

it=1;
restart = 0;
conv=0;

%%%%%%%%%%%%%%%%%%
% MAIN LOOP OF EM
%%%%%%%%%%%%%%%%%%

while(it<niter) %beginning of the loop    
    %%%%%%%%%%%%%%%%%
    % M STEP
    %%%%%%%%%%%%%%%%%
    
    for k=1:K
        logom0(:,k) = log(c.p(k)) + logvalue(c.densities{k},x);
    end
    %log-likelihood  
    ll(it) = sum(log(sum(exp(logom0')+1e-320)));
    if display==2
        disp([num2str(it) ' ll = ' num2str(ll(it))]); %display the iterations of the EM algorithm
    end
    if (it>1 & abs(ll(it)-ll(it-1))<epsi*n) | restart>maxrestart;
        conv=1;
        break      
    end
    
    %normalization of the rows to obtain probabilities of belonging to each class for every data
    om = exp(logom0);
    om = om./(sum(om,2)*ones(1,K));

    %%%%%%%%%%%%%%%%%
    % CONSTRAINT
    %%%%%%%%%%%%%%%%%

    for g=1:NL
        temp = om(indsL{g},1:K-1);
        temp2 = temp./(1-temp+eps);
        om(indsL{g},1:K-1) = temp2./(ones(numL(g),1)*(1+sum(temp2,1)));
    end
    om(:,K) = 1 - sum(om(:,1:K-1),2);
    
    %%%%%%%%%%%%%%%%%
    % M STEP
    %%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    counts = sum(om);
    c.p = max(minprop,counts/n);
    c.p = c.p/sum(c.p);
    
    for k=1:K
            if counts(k)<mincounts & restart <5 %if few data in a cluster                
                c.densities{k} = learn(c.densities{k},x,max(eps,-min(logom0,[],2)));
                restart = restart+1;
                it=1;
                %counts(k+1:end)=counts(k+1:end)+mincounts;
                break
            else
                c.densities{k} = learn(c.densities{k},x,om(:,k));
            end                
    end
    
    if ~isempty(c.commonparams)  %dependents parameters       
        c.densities = mixcommonparams(c.densities{1},c.densities(2:end),c.commonparams,c.p);
    end

    it=it+1;
end 


if display
    %fprintf('d=%d,n=%d,K=%d==>np=%d\n',d,n,K,n_params(c));
    if restart>maxrestart
        disp(['EM cannot convergence due to infinite maximum likelihood (' num2str(K) ' groups)']);        
    elseif conv 
        disp(['EM convergence in ' num2str(it-1) ' iterations (' num2str(K) ' groups)']);
    else
        disp(['EM stopped before convergence in ' num2str(niter) ' iterations (' num2str(K) ' groups)']);
    end
end
