function c=mixtdens(densities,K,varargin)
% MIXTDENS Mixture distribution (probabilistic clustering model)
%
% c= MIXTDENS creates a empty mixture of gaussian densities with 1 cluster
%
% c= MIXTDENS(dens,K) creates a mixture distribution with K clusters.
% Each cluster is modelled with objects of class dens. Use MIXTDENS/LEARN 
% method to estimate the parameters of these clusters. For example, if x
% is a nxd data matrix, 
%       c=learn(mixtdens(diagdens,3)) r
% will return a mixture distribution of dimension d with parameters learned 
% by EM algorithm.
%
% c= MIXTDENS(carray) creates a mixture distribution object with 
% K=length(carray) clusters. Each cluster is defined by the distribution
% carray{i}, i=1:K. All the densities must be density objects of the same 
% dimension d=GETDIM(carray{i}). For example 
%   dens= {gaussdens('m',[1;1],'var',eye(2)),gaussdens('m',[0;5],'var',eye(2))}
%   c = mixtdens(dens)
% returns in c a mixture distribution with 2 clustes of means ([1;1] and [0;5]) 
% and identity covariance matrix.
%
% c= MIXTDENS(...,'prop',p) set the proportions of the clusters to p
%
% c= MIXTDENS(...,'commonparams','var')
%       forces the variance parameters to be equals for each distribution
%
% Examples
% --------
% % empty mixture distribution with 2 clusters
% c = mixtdens; 
%
% % mixture of 4 gaussian distributions with identity covariance matrix
% c = mixtdens(balldens,4) 
%
% % mixture of full-covariance gaussian and spherical gaussian with specified parameters
% c = mixtdens({gaussdens('m',[1;2],'var',[1 1;1 4]),balldens('m',[2,5],'var',2)})
%
% % 4 gaussian distributions in 1 dimension with proportions [.1 .1 .1 .7]
% % with means 1, 2, 3 and 4 and common variance equal to 1
% c = mixtdens(gaussdens,'commonparams','var','p',[.1 .1 .1 .7],'m',[1 2 3 4])
%
%
% See also DENSITY, MIXTDENS/SET, MIXTDENS/LEARN, DENSITYCL, 
%   CLUSTERING, UNSUPERVISED, STATLEARN


if nargin<1 %no argument
    densities = {gaussdens,gaussdens};
    K=2;
end

if nargin<2 %K is not specified
    if isnumeric(densities)
        K = densities;        
        densities = {gaussdens};
        densities = densities(ones(1,K));
    elseif iscell(densities)
        K = length(densities);
    else
        K=2;
    end
end

if isstr(K) % no K specified
    varargin={K,varargin{:}};
    K = length(densities);
elseif length(K)>1
    Ks=K;
    K = length(Ks);
    c=cell(1,K);
    for k=1:K
        c{k} = mixtdens(densities,Ks(k),varargin);
    end
    return        
end

%now the variables densities, K and varagin exist
s = struct('p',[ones(1,K)/K],'densities',[],'commonparams',[],'knownp',0);
c = class(s,'mixtdens',density);
c.densities = densities;



if length(varargin)>0
    if iscell(varargin{1})
        varargin = varargin{1};
    end
    c = set(c,varargin);
else    
    if ~iscell(densities)
        densities = {densities};
        c.densities = densities(ones(1,K));    
    end    
    c = setdim(c,getdim(densities{1}));    
end




