function c=pcadens(varargin)
%PCADENS Defines a distribution that concentrates on a small linear subspace
%   
% This function represents a gaussian distribution whose covariance matrix has the
% form 
%       S = t_1*v_1*v_1' + ... + t_r*v_r*v_r' + e*Id
% where v_1, ..., v_r are d-dimensional orthonormal vectors representing the
% directions of high-variance (the variances in such directions are the t_i). 
% e is the variance in the subspace orthogonal to the
% vectors v_i, i=1,...,r. 
%   In matrix notation, we get S = V*D*V' + e*Id
%   where V is the d*r matrix of vectors v. In the parameters V will be denoted 
%   "proj_mat", because it corresponds to a othogonal linear projection operator
%   e will be denoted by "off_std" (standard deviation of the noise)
%
% PCA disctribution is interesting due to its low-parametrization. For example, if 
% r=1, then the number of parameters is 2*d+2, very closed to the 2*d parameters of 
% the diagonal covariance parametrization.
%
% c = PCADENS creates an empty PCADENS object
% c = PCADENS(...,'r',r,...) number of principal components
% c = PCADENS(...,'m',m,...) mean of the density
% c = PCADENS(...,'proj_mat',V,...) dxr matrix of principal components
% c = PCADENS(...,'lambda',lbd,...) vector of the variances in the principal directions (minus the noise variance)
% c = PCADENS(...,'noise',s,...) scalar value equal to the off-variance,
%   i.e. the variance in the orthogonal space of the principal components
%
% See also GAUSSDENS, DENSITY, UNSUPERVISED, STATLEARN

s = struct('m',[],'proj_mat',1,'lambda',[],'noise',[]);
c = class(s,'pcadens',density);

if nargin>0
    c=set(c,varargin);
end

