function [m,affect,i,radius] = kmeanslight(x,K,m,niter)
%KMEANSLIGHT fast K-means algorithm
%
% m = KMEANSLIGHT(x,K,m0,niter)
%   uses the K-means algorithm with the euclidian distance to set the 
% centres of a cluster model. The matrix x represents the data which 
% is clustered, each row corresponding to a data point. 
%
% Initialization:
% m0 represent the initial means (dxK matrix). If not specified,
% m0 = x(1:K,:);
% 
% Algorithm:
% The sum of squares error function (euclidian distance) is used.
% The points at which a local minimum is achieved is returned as columns of
% m. The algorithm stops as soon as the means do not change or
% niter (the maximum number of iterations, 20 by default) is reached.
% If a cluster i desappearing, a new mean is choosed on the data point having
% a maximum distance from the actual means.
% 
%
% [m,index] = KMEANSLIGHT(x,k,m0,niter)
%   also gives affectations in a nx1 vector. If m0 is empty, the 
% previous initialization is used.
%
% [m,index,nit] = KMEANSLIGHT(x,k,m0,niter)
%   also gives the number of iterations before termination
%
% [m,index,nit] = KMEANSLIGHT(x,k,m0,niter)
%   also gives the radius around each cluster
%
% example:
% figure;clf;
% x=[rand(200,2)];
% subplot(221),colplot(x);title('data points');
% [m,a,it] = kmeanslight(x,4,[],1);
% subplot(222),colplot(x,a);title([num2str(it) ' iterations']);
% [m,a,it] = kmeanslight(x,4,[],5);
% subplot(223),colplot(x,a);title([num2str(it) ' iterations']);
% [m,a,it] = kmeanslight(x,4,[],20);
% subplot(224),colplot(x,a);title([num2str(it) ' iterations']);
%
% See also KMEANS, PARTIALKMEANS, MIXTDENS, COLPLOT

if nargin<2
   K=2;
else
   if K==1
      m = mean(x)';
      affect=ones(1,size(x,1));    
      return
   end
end
if nargin<4
   niter = 20;
end

[n,d] = size(x);

pos = 0; 

if nargin<3 | isempty(m)
	m = x(mod(0:K-1,n)+1,:)';
end


MX = sum((x.^2),2)*ones(1,K);       

% Main loop of algorithm
for i=1:niter
    % Save old centres to check for termination
    old_m = m;
    
    % Calculate posteriors based on existing centres
    d2 = MX + ...
        ones(n, 1) * sum((m.^2),1) - ...
        2.*(x*m);
    
    % Assign each point to nearest centre
    [minvals, affect] = min(d2',[],1);
    [counts,ord] = sort(histc(affect,1:K));
    for j=ord        
        I = find(affect==j);
        if length(I)<=0
            [mini,ind] = max(minvals);
            m(:,j) = x(ind,:)';
            affect(ind) = j;
            minvals(ind) = 0;            
        else            
            m(:,j) = mean(x(I,:),1)';
        end
    end        
    
    % Test for termination
    if max(max(abs(m - old_m))) < eps
        break;
    end
end    


if nargout>3
	radius = zeros(1,K);
    for i=1:K
        dists2 = d2(find(affect==i),i);
        if isempty(dists2)
            radius(i) = eps;
        else
            radius(i) = sqrt(mean(sum(dists2,2),1));
        end
    end
end
