function [Z,V,ov,m] = reducedim(X,y,dim,xtored)
%REDUCEDIM linear dimensionality reduction by LDA and PCA
%
% Z = reducedim(X,y,d)
% Find the d first optimal Fisher discriminant directions, 
% projects the data onto them and return it in matrix Z of
% size nxd (X is a nxp matrix).
% If there is not enough groups to fill the K-1 discriminant directions,
% then the remaining directions are filled by the fist principal directions (by ppca function).
%
% [Z,V] = reducedim(X,y,d)
% also return the projection matrix V such that
% - V is an orthonormal basis
% - Z is computed by Z=X*V
%
% [Z,V,ov] = reducedim(X,y,d)
% also return the off-variance value (mean of the remaing eigenvalues
%
% [Z,V,ov,m] = reducedim(X,y,d)
% center the Z values and return the mean m of X in m
%
% [Z,V] = reducedim(X,y,d,xtored)
% specify in xtored the data that are reduced. The reduction is calculated from X and cl.
%
% [Z,V] = reducedim(X,d)
% where d is a scalr is a PCA dimensionality reduction (by ppca function).
%
% Used by LDAPLOT
% Uses LBL2CELL, CELL2LBL, PPCA
%


opts.disp=0;
[n,d] = size(X);

if nargin<2
    y=[];
end
if nargin<3
    dim = d;   
end
if nargin<4
    xtored=X;
end


if nargin>=2 & length(y)==1
    if isempty(y)
        dim = d;
    else
        dim = y;
    end
    y=[];
end

if dim>=d | dim==0
    Z = xtored;
    V = eye(d);
    ov = 0;
    m=mean(X)';
    return
end

if isempty(dim) | dim==0
    error('Invalid dimension');
end

if iscell(X)
    celldata=  X;
    [X,y] = cell2lbl(X);
    K = length(celldata);
else
    celldata = lbl2cell(X,y);
    K = length(celldata);
end

if ~isempty(y) | K>1

    temp=cumsum(histc(y,1:max(y))~=0);
    y=temp(y);
    
    ind = cell(1,K);
    ndesc = zeros(1,K);
    M = zeros(K,size(X,2));
    y=y-min(y)+1;
    
    for i=1:K
        if isempty(celldata{i})
            error(['class ' num2str(i) ' is empty']);
        end
        ind{i} = find(y==i);
        ndesc(i) = length(ind{i});
        M(i,:) = mean(X(ind{i},:));        
    end

    W = cov(X-M(y,:));
%    W(sub2ind([d,d],1:d,1:d)) = max(W(sub2ind([d,d],1:d,1:d)),eps);    
%    U=chol(W);
%    [Q,T] = qr(U);
%    T(sub2ind([d,d],1:d,1:d)) = max(diag(T),eps);    
%    transW = inv(T');
    transW = chol(inv(W+eye(d)*1e-3))';
    
    B = cov(M*transW);    
    [Vt,D] = eigs(B,dim,'LA',opts);

    diago = diag(D)';
    [indposeig] = find(diago>10*eps);
    
    indnegeig = setdiff(1:d,indposeig);
    V = transW*Vt(:,indposeig);
    V = V./(ones(d,1)*sqrt(sum(V.^2)));
    
    nbnulldims = dim-length(indposeig);
    if nbnulldims>0
        Xr = X*(eye(d)-(V./repmat(sum(V.^2),d,1))*V');
        [r,U,lambda] = ppca(cov(Xr),nbnulldims);
        V=[V,U];
%        disp(['last ' num2str(nbnulldims) ' dimension(s) filled by PCA directions']);
        D = diag([diago(indposeig),lambda(:)']);
    end
    
else
    if (size(X,2)==1)
        V = 1;
        D = 1;
    else
        [V,D]=eigs(cov(X),dim,'LA',opts);
    end
end

%tranformation of the coordinates


if nargout>=4
    m=mean(X)';
    Z = (xtored-ones(size(xtored,1),1)*m')*V;        
else
    Z = xtored*V;
end



if nargout>=3
    if d>dim
        ov = (sum(std(X).^2) - sum(std(Z).^2))/(d-dim);
    else 
        ov=0;
    end
end

