function [var, U, lambda] = ppca(x, ppca_dim)
%PPCA	Probabilistic Principal Components Analysis
%
%	Description
%	 [VAR, U, LAMBDA] = PPCA(X, PPCA_DIM) computes the principal
%	component subspace U of dimension PPCA_DIM using a centred covariance
%	matrix X. The variable VAR contains the off-subspace variance (which
%	is assumed to be spherical), while the vector LAMBDA contains the
%	variances of each of the principal components.  This is computed
%	using the eigenvalue and eigenvector  decomposition of X.
%
%	See also
%	EIGDEC, PCA
%

%	Copyright (c) Ian T Nabney (1996-9), Guillaume Bouchard (2003)


[ndata, data_dim] = size(x);
if data_dim==1
    var=cov(x);
    U = 1;
    lambda=0;
    return
end
k = zeros(1, ppca_dim);
% Assumes that x is centred and responsibility weighted
% covariance matrix
[l Utemp] = eigdec(x, data_dim);
% Zero any negative eigenvalues (caused by rounding)
l(l<0) = 0;
% Now compute the sigma squared values for all possible values
% of q
s2_temp = cumsum(l(end:-1:1))./[1:data_dim]';
% If necessary, reduce the value of q so that var is at least
% eps * largest eigenvalue
q_temp = min([ppca_dim; data_dim-min(find(s2_temp/(l(1)+eps) > eps))]);
if q_temp ~= ppca_dim
  fprintf(1, 'Bad conditioning : q is %d instead of ppca_dim: %d\n', q_temp,ppca_dim);
  var = l(1)/data_dim;
  if q_temp == 0
    % All the latent dimensions have disappeared, so we are
    % just left with the noise model
    lambda = var*ones(1, ppca_dim);
  end
  lambda(q_temp+1:ppca_dim) = var;
end
if q_temp
  U = Utemp(:, 1:ppca_dim);
  var = mean(l(q_temp+1:end));
  lambda(1:q_temp) = l(1:q_temp);
end

